@@ -98,7 +98,9 @@ daemonize ()
             open ("/dev/null", O_WRONLY, 0666);
             open ("/dev/null", O_WRONLY, 0666);
 
-            chdir ("/tmp");
+            if (chdir ("/tmp") < 0)
+                warn ("couldn't change to /tmp directory");
+            
             return; /* Child process returns */
         };
     }
@@ -180,15 +182,23 @@ prepare_logging ()
 
 int main(int argc, char* argv[])
 {
-    SeahorseOperation *op;
     GOptionContext *octx = NULL;
+    GError *error = NULL;
 
+    g_thread_init (NULL);
+    
     seahorse_secure_memory_init ();
     
     octx = g_option_context_new ("");
     g_option_context_add_main_entries (octx, options, GETTEXT_PACKAGE);
 
-    gtk_init_with_args (&argc, &argv, _("Encryption Daemon (Seahorse)"), (GOptionEntry *) options, GETTEXT_PACKAGE, NULL);
+    if (!gtk_init_with_args (&argc, &argv, _("Encryption Daemon (Seahorse)"), 
+                             (GOptionEntry *)options, GETTEXT_PACKAGE, &error)) {
+	    g_printerr ("seahorse-daemon: %s\n", error->message);
+	    g_error_free (error);
+	    exit (1);
+    }
+	    
 
     /* 
      * All functions after this point have to print messages 
@@ -222,8 +232,7 @@ int main(int argc, char* argv[])
     seahorse_ssh_module_init ();
 #endif
     
-    op = seahorse_context_refresh_local (NULL);
-    g_object_unref (op);
+    seahorse_context_refresh_auto (NULL);
     
     /* Initialize the various daemon components */
     seahorse_dbus_server_init ();
