@@ -33,10 +33,11 @@
 #include "seahorse-marshal.h"
 #include "seahorse-servers.h"
 #include "seahorse-transfer-operation.h"
+#include "seahorse-unknown.h"
 #include "seahorse-unknown-source.h"
-#include "seahorse-unknown-key.h"
 #include "seahorse-util.h"
 
+#include "common/seahorse-bind.h"
 #include "common/seahorse-registry.h"
 
 #ifdef WITH_PGP
@@ -50,6 +51,7 @@ enum {
     ADDED,
     REMOVED,
     CHANGED,
+    REFRESHING,
     LAST_SIGNAL
 };
 
@@ -74,6 +76,7 @@ struct _SeahorseContextPrivate {
     GHashTable *objects_by_source;          /* See explanation above */
     GHashTable *objects_by_type;            /* See explanation above */
     guint notify_id;                        /* Notify for GConf watch */
+    SeahorseMultiOperation *refresh_ops;    /* Operations for refreshes going on */
     SeahorseServiceDiscovery *discovery;    /* Adds sources from DNS-SD */
 };
 
@@ -103,7 +106,10 @@ seahorse_context_class_init (SeahorseContextClass *klass)
                 NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, SEAHORSE_TYPE_OBJECT);    
     signals[CHANGED] = g_signal_new ("changed", SEAHORSE_TYPE_CONTEXT, 
                 G_SIGNAL_RUN_FIRST, G_STRUCT_OFFSET (SeahorseContextClass, changed),
-                NULL, NULL, seahorse_marshal_VOID__OBJECT_UINT, G_TYPE_NONE, 2, SEAHORSE_TYPE_OBJECT, G_TYPE_UINT);    
+                NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, SEAHORSE_TYPE_OBJECT);    
+    signals[REFRESHING] = g_signal_new ("refreshing", SEAHORSE_TYPE_CONTEXT, 
+                G_SIGNAL_RUN_FIRST, G_STRUCT_OFFSET (SeahorseContextClass, refreshing),
+                NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, SEAHORSE_TYPE_OPERATION);    
 }
 
 /* init context, private vars, set prefs, connect signals */
@@ -125,7 +131,6 @@ seahorse_context_init (SeahorseContext *sctx)
     
     /* The context is explicitly destroyed */
     g_object_ref (sctx);
-    
 }
 
 static void
@@ -171,6 +176,10 @@ seahorse_context_dispose (GObject *gobject)
     g_slist_free (sctx->pv->sources);
     sctx->pv->sources = NULL;
     
+    if (sctx->pv->refresh_ops)
+	    g_object_unref (sctx->pv->refresh_ops);
+    sctx->pv->refresh_ops = NULL;
+    
     G_OBJECT_CLASS (seahorse_context_parent_class)->dispose (gobject);
 }
 
@@ -190,6 +199,7 @@ seahorse_context_finalize (GObject *gobject)
     g_assert (sctx->pv->sources == NULL);
     g_assert (sctx->pv->auto_sources == NULL);
     g_assert (sctx->pv->discovery == NULL);
+    g_assert (sctx->pv->refresh_ops == NULL);
     g_free (sctx->pv);
     
     G_OBJECT_CLASS (seahorse_context_parent_class)->finalize (gobject);
@@ -203,7 +213,7 @@ seahorse_context_for_app (void)
 }
    
 SeahorseContext*
-seahorse_context_new (guint flags, guint ktype)
+seahorse_context_new (guint flags)
 {
 	SeahorseContext *sctx = g_object_new (SEAHORSE_TYPE_CONTEXT, NULL);
     
@@ -211,15 +221,6 @@ seahorse_context_new (guint flags, guint ktype)
 	    sctx->is_daemon = TRUE;
     
 	if (flags & SEAHORSE_CONTEXT_APP) {
-		
-		GList *l, *types;
-		
-		types = seahorse_registry_find_types (NULL, "key-source", "local", NULL);
-		for (l = types; l; l = g_list_next (l)) {
-			SeahorseSource *src = g_object_new (GPOINTER_TO_UINT (l->data), NULL);
-			seahorse_context_take_source (sctx, src);
-		}
-		g_list_free (types);
 
 		/* DNS-SD discovery */    
 		sctx->pv->discovery = seahorse_service_discovery_new ();
@@ -262,27 +263,50 @@ seahorse_context_destroy (SeahorseContext *sctx)
         app_context = NULL;
 }
 
+static gboolean                
+take_source (SeahorseContext *sctx, SeahorseSource *sksrc)
+{
+	SeahorseOperation *operation;
+	
+	g_return_val_if_fail (SEAHORSE_IS_SOURCE (sksrc), FALSE);
+	if (!g_slist_find (sctx->pv->sources, sksrc)) {
+		sctx->pv->sources = g_slist_append (sctx->pv->sources, sksrc);
+		if (seahorse_source_get_location (sksrc) == SEAHORSE_LOCATION_LOCAL && 
+		    sctx->pv->refresh_ops != NULL) {
+			operation = seahorse_source_load (sksrc);
+			g_return_val_if_fail (operation, TRUE);
+			seahorse_multi_operation_take (sctx->pv->refresh_ops, operation);
+			g_signal_emit (sctx, signals[REFRESHING], 0, sctx->pv->refresh_ops);
+		}
+		return TRUE;
+	}
+	
+	return FALSE;
+}
+
 void                
 seahorse_context_take_source (SeahorseContext *sctx, SeahorseSource *sksrc)
 {
-    g_return_if_fail (SEAHORSE_IS_CONTEXT (sctx));
-    g_return_if_fail (SEAHORSE_IS_SOURCE (sksrc));
+	g_return_if_fail (SEAHORSE_IS_SOURCE (sksrc));
+    
+	if (!sctx)
+		sctx = seahorse_context_for_app ();
+	g_return_if_fail (SEAHORSE_IS_CONTEXT (sctx));
 
-    if (!g_slist_find (sctx->pv->sources, sksrc))
-        sctx->pv->sources = g_slist_append (sctx->pv->sources, sksrc);
+	take_source (sctx, sksrc);
 }
 
 void
 seahorse_context_add_source (SeahorseContext *sctx, SeahorseSource *sksrc)
 {
-    g_return_if_fail (SEAHORSE_IS_CONTEXT (sctx));
-    g_return_if_fail (SEAHORSE_IS_SOURCE (sksrc));
-
-    if (g_slist_find (sctx->pv->sources, sksrc))
-        return;
+	g_return_if_fail (SEAHORSE_IS_SOURCE (sksrc));
     
-    sctx->pv->sources = g_slist_append (sctx->pv->sources, sksrc);
-    g_object_ref (sksrc);
+	if (!sctx)
+		sctx = seahorse_context_for_app ();
+	g_return_if_fail (SEAHORSE_IS_CONTEXT (sctx));
+
+	if (take_source (sctx, sksrc))
+		g_object_ref (sksrc);
 }
     
 void
@@ -290,9 +314,12 @@ seahorse_context_remove_source (SeahorseContext *sctx, SeahorseSource *sksrc)
 {
     GList *l, *objects;
     
-    g_return_if_fail (SEAHORSE_IS_CONTEXT (sctx));
     g_return_if_fail (SEAHORSE_IS_SOURCE (sksrc));
 
+    if (!sctx)
+        sctx = seahorse_context_for_app ();
+    g_return_if_fail (SEAHORSE_IS_CONTEXT (sctx));
+
     if (!g_slist_find (sctx->pv->sources, sksrc)) 
         return;
 
@@ -313,12 +340,14 @@ seahorse_context_find_source (SeahorseContext *sctx, GQuark ktype,
     SeahorseSource *ks;
     GSList *l;
     
+    if (!sctx)
+        sctx = seahorse_context_for_app ();
     g_return_val_if_fail (SEAHORSE_IS_CONTEXT (sctx), NULL);
 
     for (l = sctx->pv->sources; l; l = g_slist_next (l)) {
         ks = SEAHORSE_SOURCE (l->data);
         
-        if (ktype != SKEY_UNKNOWN && 
+        if (ktype != SEAHORSE_TAG_INVALID && 
             seahorse_source_get_ktype (ks) != ktype)
             continue;
         
@@ -330,7 +359,7 @@ seahorse_context_find_source (SeahorseContext *sctx, GQuark ktype,
     }
     
     /* If we don't have an unknown source for this type, create it */
-    if (location == SEAHORSE_LOCATION_MISSING && location != SKEY_UNKNOWN) {
+    if (location == SEAHORSE_LOCATION_MISSING && location != SEAHORSE_TAG_INVALID) {
         ks = SEAHORSE_SOURCE (seahorse_unknown_source_new (ktype));
         seahorse_context_add_source (sctx, ks);
         return ks;
@@ -347,12 +376,14 @@ seahorse_context_find_sources (SeahorseContext *sctx, GQuark ktype,
     GSList *sources = NULL;
     GSList *l;
     
+    if (!sctx)
+        sctx = seahorse_context_for_app ();
     g_return_val_if_fail (SEAHORSE_IS_CONTEXT (sctx), NULL);
 
     for (l = sctx->pv->sources; l; l = g_slist_next (l)) {
         ks = SEAHORSE_SOURCE (l->data);
         
-        if (ktype != SKEY_UNKNOWN && 
+        if (ktype != SEAHORSE_TAG_INVALID && 
             seahorse_source_get_ktype (ks) != ktype)
             continue;
         
@@ -374,9 +405,12 @@ seahorse_context_remote_source (SeahorseContext *sctx, const gchar *uri)
     gchar *ks_uri;
     GSList *l;
     
-    g_return_val_if_fail (SEAHORSE_IS_CONTEXT (sctx), NULL);
     g_return_val_if_fail (uri && *uri, NULL);
 
+    if (!sctx)
+        sctx = seahorse_context_for_app ();
+    g_return_val_if_fail (SEAHORSE_IS_CONTEXT (sctx), NULL);
+
     for (l = sctx->pv->sources; l; l = g_slist_next (l)) {
         ks = SEAHORSE_SOURCE (l->data);
         
@@ -411,16 +445,9 @@ seahorse_context_remote_source (SeahorseContext *sctx, const gchar *uri)
 
 
 static void
-object_changed (SeahorseObject *sobj, SeahorseKeyChange change, SeahorseContext *sctx)
+object_notify (SeahorseObject *sobj, GParamSpec *spec, SeahorseContext *sctx)
 {
-    g_signal_emit (sctx, signals[CHANGED], 0, sobj, change);
-}
-
-static void
-object_destroyed (SeahorseObject *sobj, SeahorseContext *sctx)
-{
-    /* When objects are destroyed elsewhere */
-    seahorse_context_remove_object (sctx, sobj);
+	g_signal_emit (sctx, signals[CHANGED], 0, sobj);
 }
 
 static gpointer                 
@@ -494,6 +521,10 @@ setup_objects_by_type (SeahorseContext *sctx, SeahorseObject *sobj, gboolean add
 void
 seahorse_context_add_object (SeahorseContext *sctx, SeahorseObject *sobj)
 {
+    if (!sctx)
+        sctx = seahorse_context_for_app ();
+    g_return_if_fail (SEAHORSE_IS_CONTEXT (sctx));
+
     g_object_ref (sobj);
     seahorse_context_take_object (sctx, sobj);
 }
@@ -503,10 +534,11 @@ seahorse_context_take_object (SeahorseContext *sctx, SeahorseObject *sobj)
 {
     gpointer ks;
     
+    if (!sctx)
+        sctx = seahorse_context_for_app ();
     g_return_if_fail (SEAHORSE_IS_CONTEXT (sctx));
     g_return_if_fail (SEAHORSE_IS_OBJECT (sobj));
-    g_return_if_fail (sobj->_id != 0);
-    g_return_if_fail (!sobj->attached_to);
+    g_return_if_fail (seahorse_object_get_id (sobj) != 0);
     
     ks = hashkey_by_source (seahorse_object_get_source (sobj), 
                             seahorse_object_get_id (sobj));
@@ -515,19 +547,21 @@ seahorse_context_take_object (SeahorseContext *sctx, SeahorseObject *sobj)
 
     g_object_ref (sobj);
 
-    sobj->attached_to = sctx;
+    g_object_set (sobj, "context", sctx, NULL);
     g_hash_table_replace (sctx->pv->objects_by_source, ks, sobj);
     setup_objects_by_type (sctx, sobj, TRUE);
     g_signal_emit (sctx, signals[ADDED], 0, sobj);
     g_object_unref (sobj);
     
-    g_signal_connect (sobj, "changed", G_CALLBACK (object_changed), sctx);
-    g_signal_connect (sobj, "destroy", G_CALLBACK (object_destroyed), sctx);
+    g_signal_connect (sobj, "notify", G_CALLBACK (object_notify), sctx);
 }
 
 guint
 seahorse_context_get_count (SeahorseContext *sctx)
 {
+    if (!sctx)
+        sctx = seahorse_context_for_app ();
+    g_return_val_if_fail (SEAHORSE_IS_CONTEXT (sctx), 0);
     return g_hash_table_size (sctx->pv->objects_by_source);
 }
 
@@ -537,6 +571,8 @@ seahorse_context_get_object (SeahorseContext *sctx, SeahorseSource *sksrc,
 {
     gconstpointer k;
     
+    if (!sctx)
+        sctx = seahorse_context_for_app ();
     g_return_val_if_fail (SEAHORSE_IS_CONTEXT (sctx), NULL);
     g_return_val_if_fail (SEAHORSE_IS_SOURCE (sksrc), NULL);
     
@@ -545,52 +581,104 @@ seahorse_context_get_object (SeahorseContext *sctx, SeahorseSource *sksrc,
 }
 
 typedef struct _ObjectMatcher {
-    
-    SeahorseObjectPredicate *kp;
-    gboolean many;
-    GList *objects;
-    
+	SeahorseObjectPredicate *kp;
+	gboolean many;
+	SeahorseObjectFunc func;
+	gpointer user_data;
 } ObjectMatcher;
 
 gboolean
 find_matching_objects (gpointer key, SeahorseObject *sobj, ObjectMatcher *km)
 {
-    if (km->kp && seahorse_object_predicate_match (km->kp, SEAHORSE_OBJECT (sobj)))
-        km->objects = g_list_prepend (km->objects, sobj);
+	gboolean matched;
+	
+	if (km->kp && seahorse_object_predicate_match (km->kp, SEAHORSE_OBJECT (sobj))) {
+		matched = TRUE;
+		(km->func) (sobj, km->user_data);
+	}
 
-    /* Terminate search */
-    if (!km->many && km->objects)
-        return TRUE;
+	/* Terminate search */
+	if (!km->many && matched)
+		return TRUE;
 
-    /* Keep going */
-    return FALSE;
+	/* Keep going */
+	return FALSE;
 }
 
-GList*             
-seahorse_context_get_objects (SeahorseContext *sctx, SeahorseSource *sksrc)
+static void
+add_object_to_list (SeahorseObject *object, gpointer user_data)
 {
-    SeahorseObjectPredicate kp;
-    ObjectMatcher km;
+	GList** list = (GList**)user_data;
+	*list = g_list_prepend (*list, object);
+}
 
-    g_return_val_if_fail (SEAHORSE_IS_CONTEXT (sctx), NULL);
-    g_return_val_if_fail (sksrc == NULL || SEAHORSE_IS_SOURCE (sksrc), NULL);
+GList*
+seahorse_context_find_objects_full (SeahorseContext *self, SeahorseObjectPredicate *pred)
+{
+	GList *list = NULL;
+	ObjectMatcher km;
 
-    memset (&kp, 0, sizeof (kp));
-    memset (&km, 0, sizeof (km));
-    
-    km.kp = &kp;
-    km.many = TRUE;
-    kp.source = sksrc;
-    
-    g_hash_table_find (sctx->pv->objects_by_source, (GHRFunc)find_matching_objects, &km);
-    return km.objects;
+	if (!self)
+		self = seahorse_context_for_app ();
+	g_return_val_if_fail (SEAHORSE_IS_CONTEXT (self), NULL);
+	g_return_val_if_fail (pred, NULL);
+
+	memset (&km, 0, sizeof (km));
+	km.kp = pred;
+	km.many = TRUE;
+	km.func = add_object_to_list;
+	km.user_data = &list;
+
+	g_hash_table_find (self->pv->objects_by_source, (GHRFunc)find_matching_objects, &km);
+	return list; 
+}
+
+void
+seahorse_context_for_objects_full (SeahorseContext *self, SeahorseObjectPredicate *pred,
+                                   SeahorseObjectFunc func, gpointer user_data)
+{
+	ObjectMatcher km;
+
+	if (!self)
+		self = seahorse_context_for_app ();
+	g_return_if_fail (SEAHORSE_IS_CONTEXT (self));
+	g_return_if_fail (pred);
+	g_return_if_fail (func);
+
+	memset (&km, 0, sizeof (km));
+	km.kp = pred;
+	km.many = TRUE;
+	km.func = func;
+	km.user_data = user_data;
+
+	g_hash_table_find (self->pv->objects_by_source, (GHRFunc)find_matching_objects, &km);
+}
+
+GList*
+seahorse_context_get_objects (SeahorseContext *self, SeahorseSource *source)
+{
+	SeahorseObjectPredicate pred;
+
+	if (!self)
+		self = seahorse_context_for_app ();
+	g_return_val_if_fail (SEAHORSE_IS_CONTEXT (self), NULL);
+	g_return_val_if_fail (source == NULL || SEAHORSE_IS_SOURCE (source), NULL);
+
+	seahorse_object_predicate_clear (&pred);
+	pred.source = source;
+	
+	return seahorse_context_find_objects_full (self, &pred);
 }
 
 SeahorseObject*        
 seahorse_context_find_object (SeahorseContext *sctx, GQuark id, SeahorseLocation location)
 {
     SeahorseObject *sobj; 
-    
+
+    if (!sctx)
+        sctx = seahorse_context_for_app ();
+    g_return_val_if_fail (SEAHORSE_IS_CONTEXT (sctx), NULL);
+
     sobj = (SeahorseObject*)g_hash_table_lookup (sctx->pv->objects_by_type, GUINT_TO_POINTER (id));
     while (sobj) {
         
@@ -614,6 +702,10 @@ seahorse_context_find_objects (SeahorseContext *sctx, GQuark ktype,
 {
     SeahorseObjectPredicate pred;
     memset (&pred, 0, sizeof (pred));
+
+    if (!sctx)
+        sctx = seahorse_context_for_app ();
+    g_return_val_if_fail (SEAHORSE_IS_CONTEXT (sctx), NULL);
     
     pred.tag = ktype;
     pred.usage = usage;
@@ -622,33 +714,13 @@ seahorse_context_find_objects (SeahorseContext *sctx, GQuark ktype,
     return seahorse_context_find_objects_full (sctx, &pred);
 }
 
-GList*
-seahorse_context_find_objects_full (SeahorseContext *sctx, SeahorseObjectPredicate *skpred)
-{
-    ObjectMatcher km;
-
-    g_return_val_if_fail (SEAHORSE_IS_CONTEXT (sctx), NULL);
-
-    memset (&km, 0, sizeof (km));
-    
-    km.kp = skpred;
-    km.many = TRUE;
-    
-    g_hash_table_find (sctx->pv->objects_by_source, (GHRFunc)find_matching_objects, &km);
-    return km.objects; 
-}
-
-gboolean
-seahorse_context_owns_object (SeahorseContext *sctx, SeahorseObject *sobj)
-{
-    return sobj->attached_to == sctx;
-}
-
 void 
 seahorse_context_remove_object (SeahorseContext *sctx, SeahorseObject *sobj)
 {
     gconstpointer k;
     
+    if (!sctx)
+        sctx = seahorse_context_for_app ();
     g_return_if_fail (SEAHORSE_IS_CONTEXT (sctx));
     g_return_if_fail (SEAHORSE_IS_OBJECT (sobj));
     
@@ -656,12 +728,11 @@ seahorse_context_remove_object (SeahorseContext *sctx, SeahorseObject *sobj)
                            seahorse_object_get_id (sobj));
     
     if (g_hash_table_lookup (sctx->pv->objects_by_source, k)) {
-        g_return_if_fail (sobj->attached_to == sctx);
+        g_return_if_fail (seahorse_object_get_context (sobj) == sctx);
 
         g_object_ref (sobj);
-        g_signal_handlers_disconnect_by_func (sobj, object_changed, sctx);
-        g_signal_handlers_disconnect_by_func (sobj, object_destroyed, sctx);
-        sobj->attached_to = NULL;
+        g_signal_handlers_disconnect_by_func (sobj, object_notify, sctx);
+        g_object_set (sobj, "context", NULL, NULL);
         g_hash_table_remove (sctx->pv->objects_by_source, k);
         setup_objects_by_type (sctx, sobj, FALSE);
         g_signal_emit (sctx, signals[REMOVED], 0, sobj);    
@@ -680,12 +751,16 @@ seahorse_context_remove_object (SeahorseContext *sctx, SeahorseObject *sobj)
  * 
  * Returns: the secret key that's the default key 
  */
-SeahorseKey*
+SeahorseObject*
 seahorse_context_get_default_key (SeahorseContext *sctx)
 {
     SeahorseObject *sobj = NULL;
     gchar *id;
     
+    if (!sctx)
+        sctx = seahorse_context_for_app ();
+    g_return_val_if_fail (SEAHORSE_IS_CONTEXT (sctx), NULL);
+
     /* TODO: All of this needs to take multiple key types into account */
     
     id = seahorse_gconf_get_string (SEAHORSE_DEFAULT_KEY);
@@ -696,9 +771,7 @@ seahorse_context_get_default_key (SeahorseContext *sctx)
     
     g_free (id);
     
-    if (SEAHORSE_IS_KEY (sobj))
-	    return SEAHORSE_KEY (sobj);
-    return NULL;
+    return sobj;
 }
 
 /**
@@ -712,57 +785,45 @@ seahorse_context_get_default_key (SeahorseContext *sctx)
 SeahorseServiceDiscovery*
 seahorse_context_get_discovery (SeahorseContext *sctx)
 {
+    if (!sctx)
+        sctx = seahorse_context_for_app ();
+    g_return_val_if_fail (SEAHORSE_IS_CONTEXT (sctx), NULL);
     g_return_val_if_fail (sctx->pv->discovery != NULL, NULL);
+    
     return sctx->pv->discovery;
 }
 
-SeahorseOperation*  
-seahorse_context_load_local_objects (SeahorseContext *sctx)
-{
-    SeahorseSource *ks;
-    SeahorseMultiOperation *mop = NULL;
-    SeahorseOperation *op = NULL;
-    GSList *l;
-    
-    g_return_val_if_fail (SEAHORSE_IS_CONTEXT (sctx), NULL);
-
-    for (l = sctx->pv->sources; l; l = g_slist_next (l)) {
-        ks = SEAHORSE_SOURCE (l->data);
-        
-        if (seahorse_source_get_location (ks) == SEAHORSE_LOCATION_LOCAL) {
-            if (mop == NULL && op != NULL) {
-                mop = seahorse_multi_operation_new ();
-                seahorse_multi_operation_take (mop, op);
-            }
-            
-            op = seahorse_source_load (ks, 0);
-            
-            if (mop != NULL)
-                seahorse_multi_operation_take (mop, op);
-        }
-    }   
-
-    return mop ? SEAHORSE_OPERATION (mop) : op;  
-}
-
-static gboolean 
-load_local_objects (SeahorseContext *sctx)
-{
-    SeahorseOperation *op = seahorse_context_load_local_objects (sctx);
-    g_return_val_if_fail (op != NULL, FALSE);
-    g_object_unref (op);
-    return FALSE;
-}
-
 void
-seahorse_context_load_local_objects_async (SeahorseContext *sctx)
+seahorse_context_refresh_auto (SeahorseContext *sctx)
 {
-    g_idle_add_full (G_PRIORITY_LOW, (GSourceFunc)load_local_objects, sctx, NULL);
+	SeahorseSource *ks;
+	SeahorseOperation *op = NULL;
+	GSList *l;
+    
+	if (!sctx)
+		sctx = seahorse_context_for_app ();
+	g_return_if_fail (SEAHORSE_IS_CONTEXT (sctx));
+	
+	if (!sctx->pv->refresh_ops)
+		sctx->pv->refresh_ops = seahorse_multi_operation_new ();
+
+	for (l = sctx->pv->sources; l; l = g_slist_next (l)) {
+		ks = SEAHORSE_SOURCE (l->data);
+        
+		if (seahorse_source_get_location (ks) == SEAHORSE_LOCATION_LOCAL) {
+
+			op = seahorse_source_load (ks);
+			g_return_if_fail (op);
+			seahorse_multi_operation_take (sctx->pv->refresh_ops, op);
+		}
+		
+	}
+	
+	g_signal_emit (sctx, signals[REFRESHING], 0, sctx->pv->refresh_ops);
 }
 
-
 SeahorseOperation*  
-seahorse_context_load_remote_objects (SeahorseContext *sctx, const gchar *search)
+seahorse_context_search_remote (SeahorseContext *sctx, const gchar *search)
 {
     SeahorseSource *ks;
     SeahorseMultiOperation *mop = NULL;
@@ -771,6 +832,8 @@ seahorse_context_load_remote_objects (SeahorseContext *sctx, const gchar *search
     GHashTable *servers = NULL;
     gchar *uri;
     
+    if (!sctx)
+        sctx = seahorse_context_for_app ();
     g_return_val_if_fail (SEAHORSE_IS_CONTEXT (sctx), NULL);
     
     /* Get a list of all selected key servers */
@@ -797,7 +860,7 @@ seahorse_context_load_remote_objects (SeahorseContext *sctx, const gchar *search
             g_free (uri);
         }
 
-        if (op != NULL) {
+        if (mop == NULL && op != NULL) {
             mop = seahorse_multi_operation_new ();
             seahorse_multi_operation_take (mop, op);
         }
@@ -892,6 +955,10 @@ seahorse_context_transfer_objects (SeahorseContext *sctx, GList *objects,
     GList *next, *l;
     GQuark ktype;
 
+    if (!sctx)
+        sctx = seahorse_context_for_app ();
+    g_return_val_if_fail (SEAHORSE_IS_CONTEXT (sctx), NULL);
+
     objects = g_list_copy (objects);
     
     /* Sort by key source */
@@ -970,6 +1037,10 @@ seahorse_context_retrieve_objects (SeahorseContext *sctx, GQuark ktype,
     SeahorseSource *sksrc;
     GSList *sources, *l;
     
+    if (!sctx)
+        sctx = seahorse_context_for_app ();
+    g_return_val_if_fail (SEAHORSE_IS_CONTEXT (sctx), NULL);
+
     if (!to) {
         to = seahorse_context_find_source (sctx, ktype, SEAHORSE_LOCATION_LOCAL);
         if (!to) {
@@ -1020,10 +1091,14 @@ seahorse_context_discover_objects (SeahorseContext *sctx, GQuark ktype,
     SeahorseOperation *op;
     GSList *l;
 
+    if (!sctx)
+        sctx = seahorse_context_for_app ();
+    g_return_val_if_fail (SEAHORSE_IS_CONTEXT (sctx), NULL);
+
     /* Check all the ids */
     for (l = rawids; l; l = g_slist_next (l)) {
         
-        id = seahorse_source_canonize_id (ktype, (gchar*)l->data);
+        id = seahorse_context_canonize_id (ktype, (gchar*)l->data);
         if (!id) {
             /* TODO: Try and match this partial id */
             g_warning ("invalid id: %s", (gchar*)l->data);
@@ -1096,54 +1171,39 @@ seahorse_context_discover_objects (SeahorseContext *sctx, GQuark ktype,
 }
 
 SeahorseObject*
-seahorse_context_object_from_dbus (SeahorseContext *sctx, const gchar *key, guint *uid)
+seahorse_context_object_from_dbus (SeahorseContext *sctx, const gchar *key)
 {
     SeahorseObject *sobj;
-    const gchar *t = NULL;
-    gchar *x, *alloc = NULL;
-    
-    /* Find the second colon, if any */
-    t = strchr (key, ':');
-    if (t != NULL) {
-        t = strchr (t + 1, ':');
-        if(t != NULL) {
-            key = alloc = g_strndup (key, t - key);
-            ++t;
-        }
-    }
     
     /* This will always get the most preferred key */
     sobj = seahorse_context_find_object (sctx, g_quark_from_string (key), 
                                          SEAHORSE_LOCATION_INVALID);
     
-    if (uid)
-        *uid = 0;
-        
-    /* Parse out the uid */
-    if (SEAHORSE_IS_KEY (sobj) && sobj && t) {
-        glong l = strtol (t, &x, 10);
-            
-        /* Make sure it's valid */
-        if (*x || l < 0 || l >= seahorse_key_get_num_names (SEAHORSE_KEY (sobj)))
-            sobj = NULL;
-        else if (uid)
-            *uid = (guint)l;
-    }
-    
     return sobj;
 }
 
 gchar*
-seahorse_context_object_to_dbus (SeahorseContext *sctx, SeahorseObject *sobj, guint uid)
+seahorse_context_object_to_dbus (SeahorseContext *sctx, SeahorseObject *sobj)
 {
-    return seahorse_context_id_to_dbus (sctx, seahorse_object_get_id (sobj), uid);
+    return seahorse_context_id_to_dbus (sctx, seahorse_object_get_id (sobj));
 }
 
 gchar*
-seahorse_context_id_to_dbus (SeahorseContext* sctx, GQuark id, guint uid)
+seahorse_context_id_to_dbus (SeahorseContext* sctx, GQuark id)
 {
-    if (uid == 0)
-        return g_strdup (g_quark_to_string (id));
-    else
-        return g_strdup_printf ("%s:%d", g_quark_to_string (id), uid);
+	return g_strdup (g_quark_to_string (id));
+}
+
+GQuark
+seahorse_context_canonize_id (GQuark ktype, const gchar *id)
+{
+	SeahorseCanonizeFunc canonize;
+
+	g_return_val_if_fail (id != NULL, 0);
+    
+	canonize = seahorse_registry_lookup_function (NULL, "canonize", g_quark_to_string (ktype), NULL);
+	if (!canonize) 
+		return 0;
+	
+	return (canonize) (id);
 }
