@@ -43,11 +43,11 @@
 #include "seahorse-context.h"
 
 #define SEAHORSE_TYPE_WIDGET            (seahorse_widget_get_type ())
-#define SEAHORSE_WIDGET(obj)            (GTK_CHECK_CAST ((obj), SEAHORSE_TYPE_WIDGET, SeahorseWidget))
-#define SEAHORSE_WIDGET_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), SEAHORSE_TYPE_WIDGET, SeahorseWidgetClass))
-#define SEAHORSE_IS_WIDGET(obj)         (GTK_CHECK_TYPE ((obj), SEAHORSE_TYPE_WIDGET))
-#define SEAHORSE_IS_WIDGET_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), SEAHORSE_TYPE_WIDGET))
-#define SEAHORSE_WIDGET_GET_CLASS(obj)  (GTK_CHECK_GET_CLASS ((obj), SEAHORSE_TYPE_WIDGET, SeahorseWidgetClass))
+#define SEAHORSE_WIDGET(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), SEAHORSE_TYPE_WIDGET, SeahorseWidget))
+#define SEAHORSE_WIDGET_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), SEAHORSE_TYPE_WIDGET, SeahorseWidgetClass))
+#define SEAHORSE_IS_WIDGET(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SEAHORSE_TYPE_WIDGET))
+#define SEAHORSE_IS_WIDGET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SEAHORSE_TYPE_WIDGET))
+#define SEAHORSE_WIDGET_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), SEAHORSE_TYPE_WIDGET, SeahorseWidgetClass))
 
 typedef struct _SeahorseWidget SeahorseWidget;
 typedef struct _SeahorseWidgetClass SeahorseWidgetClass;
@@ -60,7 +60,6 @@ struct _SeahorseWidget {
     gchar *name;
     
     /*< private >*/
-    GtkUIManager   *ui;
     gboolean destroying;
 };
 
@@ -78,7 +77,9 @@ SeahorseWidget*  seahorse_widget_new_allow_multiple (const gchar      *name,
 
 SeahorseWidget*  seahorse_widget_find               (const gchar      *name);
 
-GtkWidget*       seahorse_widget_get_top            (SeahorseWidget   *swidget);
+const gchar*     seahorse_widget_get_name           (SeahorseWidget   *swidget);
+
+GtkWidget*       seahorse_widget_get_toplevel       (SeahorseWidget   *swidget);
 
 GtkWidget*       seahorse_widget_get_widget         (SeahorseWidget   *swidget,
                                                      const char       *identifier);
@@ -91,15 +92,15 @@ void             seahorse_widget_set_visible        (SeahorseWidget   *swidget,
                                                      const char       *identifier,
                                                      gboolean         visible);
 
+void             seahorse_widget_set_sensitive      (SeahorseWidget   *swidget,
+                                                     const char       *identifier,
+                                                     gboolean         sensitive);
+
+void             seahorse_widget_connect_glade_signal (SeahorseWidget *swidget, 
+                                                       const char     *event,
+                                                       GtkCallback    callback,
+                                                       gpointer       userdata);
+
 void             seahorse_widget_destroy            (SeahorseWidget   *swidget);
 
-GtkWidget*       seahorse_widget_get_ui_widget      (SeahorseWidget *swidget, 
-                                                     const gchar *path);
-
-void             seahorse_widget_add_actions        (SeahorseWidget   *swidget,
-                                                     GtkActionGroup   *actions);
-
-GtkActionGroup*  seahorse_widget_find_actions       (SeahorseWidget   *swidget, 
-                                                     const gchar *name);
-
 #endif /* __SEAHORSE_WIDGET_H__ */
